// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package configuration

import (
	"github.com/r3labs/diff/v3"
)

type ChangeEvent struct {
	Changlog diff.Changelog
}

type ChangeHook interface {
	Handle(event ChangeEvent)
}

// OnChange registers a hook that is called when the configuration changes.
func (s *Settings[C]) OnChange(hook ChangeHook) *Settings[C] {
	s.hooks.change = append(s.hooks.change, hook)
	return s
}

// HasOnChangeHook returns true if there are registered hooks.
func (s *Settings[C]) HasOnChangeHook(hook ChangeHook) *Settings[C] {
	for _, h := range s.hooks.change {
		if h == hook {
			break
		}
	}
	return s
}

// RemoveOnChangeHook removes a change hook from the list of hooks.
func (s *Settings[C]) RemoveOnChangeHook(hook ChangeHook) *Settings[C] {
	for i, h := range s.hooks.change {
		if h == hook {
			s.hooks.change = append(s.hooks.change[:i], s.hooks.change[i+1:]...)
			break
		}
	}
	return s
}

func (s *Settings[C]) notifyChangeHooks(changelog diff.Changelog) *Settings[C] {
	for _, h := range s.hooks.change {
		go h.Handle(ChangeEvent{Changlog: changelog})
	}
	return s
}
