// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package configuration

import (
	"github.com/r3labs/diff/v3"
)

func (s *Settings[C]) setConfigInternal(config C, lock bool) *Settings[C] {

	var (
		changelog diff.Changelog
		err       error
	)

	if lock {
		s.Lock()
	}

	defer func() {
		if lock {
			s.Unlock()
		}
	}()

	defer func() {

		if len(changelog) > 0 {
			s.notifyChangeHooks(changelog)
		}

	}()

	errorCount := len(s.errors)
	defer func() {
		if len(s.errors) > errorCount {
			s.notifyErrorHooks()
		}
	}()

	if err := validateConfig[C](config); err != nil {
		s.errors = append(s.errors, err)
		return s
	}

	d, err := diff.NewDiffer()
	if err != nil {
		s.errors = append(s.errors, err)
		return s
	}

	d.ConvertCompatibleTypes = true
	d.AllowTypeMismatch = true

	changelog, err = d.Diff(s.config, config)
	if err != nil {
		s.errors = append(s.errors, err)
		return s
	}

	s.config = config
	return s
}

func (s *Settings[C]) SetConfig(config C) *Settings[C] {
	return s.setConfigInternal(config, true)
}
