// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package configuration

type ErrorEvent struct {
}

type ErrorHook interface {
	Handle(event ErrorEvent)
}

// OnChange registers a hook that is called when the configuration changes.
func (s *Settings[C]) OnError(hook ErrorHook) *Settings[C] {
	s.hooks.error = append(s.hooks.error, hook)
	return s
}

// HasOnChangeHook returns true if there are registered hooks.
func (s *Settings[C]) HasOnErrorHook(hook ErrorHook) *Settings[C] {
	for _, h := range s.hooks.error {
		if h == hook {
			break
		}
	}
	return s
}

// RemoveOnChangeHook removes a change hook from the list of hooks.
func (s *Settings[C]) RemoveOnErrorHook(hook ErrorHook) *Settings[C] {
	for i, h := range s.hooks.error {
		if h == hook {
			s.hooks.error = append(s.hooks.error[:i], s.hooks.error[i+1:]...)
			break
		}
	}
	return s
}

func (s *Settings[C]) notifyErrorHooks() *Settings[C] {
	for _, h := range s.hooks.error {
		go h.Handle(ErrorEvent{})
	}
	return s
}
